@@ -1,7 +1,7 @@
 /*
  * pftp -- sends files from host to host through free choosable ports
  *
- * Copyright (C) 1996-1999 Ben Schluricke
+ * Copyright (C) 1996-2000 Ben Schluricke
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,24 +33,30 @@
 #include <fcntl.h>
 #include "main.h"
 
+#if !defined __RH_CYGLINT__
 extern void set_tty(int);
+extern void create_passwd(void);
+extern void pmanager(short);
+#endif
 extern void shost(int *, int);
 extern void lhost(int *, short);
 extern void set_vars_from_pftprc(FILE *, int *, int);
 extern short get_var_from_pftprc(FILE *, const char *, char *, int);
-extern void pmanager(short);
 extern void warranty(void);
 extern short mycmp(char *, char *);
 extern void help();
 extern int printlist(char **, char *, short);
-extern void create_passwd(void);
 extern int getoffset(char *);
 extern int extract_val_d(double *, char *, int);
 extern int extract_val_i(unsigned int *, char *, int);
 extern int extract_val_li(unsigned long *, char *, int);
+extern void opt_err_handle(int);
 
-long offset[SONAME];  /* hold all offsets of the beginning of files */
+off_t offset[SONAME];  /* hold all offsets of the beginning of files */
 
+#if defined __RH_CYGLINT__
+#define NO_SUCH_OPT() fprintf(stderr, "** This option is not implemented on this operating system!\n"); exit(1);
+#endif
 
 int argum(int argc, char **argv, int *portn)
 {
@@ -59,6 +65,7 @@ int argum(int argc, char **argv, int *portn)
    int mfs=0, len=0, count=0;
    char set_netbuffer=0, set_bandwidth=0, list_filters=0;
    char *gtmp=NULL, stmp[SONAME], *envtmp=NULL;
+   char GIGABIT_ETHERNET=0;
    (*statstr)->usefilter = 0;
    *offset = -1;
 
@@ -79,33 +86,53 @@ int argum(int argc, char **argv, int *portn)
       free(blankline);
       fprintf(stderr, "This program is dedicated to Heather O'Rourke");
       fprintf(stderr, "[0m[%dH", _WINROWS_);
+#if !defined __RH_CYGLINT__
       set_tty(2); fgetc(stdin); set_tty(0);
+#else
+      sleep(1);
+#endif
       fprintf(stderr, "[?25h[H[J");
       exit(PFTP_NO_ERR);
    }
    else if (argc == 2 && !mycmp(*(argv+1), "-n")) {
+#if !defined __RH_CYGLINT__
       pmanager(0);
+#else
+      NO_SUCH_OPT();
+#endif
       exit(PFTP_NO_ERR);
    }
    else if (argc == 2 && *(*(argv+1)) == '-' && *(*(argv+1)+1) == 'N') {
       if (isdigit(*(*(argv+1)+2))) {
          (*statstr)->host_number = atoi((*(argv+1)+2));
       }
+#if !defined __RH_CYGLINT__
       pmanager(1);
+#else
+      NO_SUCH_OPT();
+#endif
       exit(PFTP_NO_ERR);
    }
    else if (argc == 2 && !mycmp(*(argv+1), "-nl")) {
       while (!*portn) {
          lhost(portn, 0);
+#if !defined __RH_CYGLINT__
          pmanager(0);
+#else
+         NO_SUCH_OPT();
+#endif
       }
    }
    else if (argc == 2 && !mycmp(*(argv+1), "-W")) {
+#if !defined __RH_CYGLINT__
       fprintf(stderr, "[?25l");
       set_tty(2);
+#endif
       warranty();
+#if !defined __RH_CYGLINT__
       set_tty(0);
       fprintf(stderr, "[?25h\r");
+#endif
       exit(PFTP_NO_ERR);
    }
    else if (argc == 1) {
@@ -130,9 +157,13 @@ int argum(int argc, char **argv, int *portn)
             FILE *fp=NULL;
             char ch='\0', str[8];
             fprintf(stderr , "Add `%s' to your host name list (Y|n)? ", *(argv+1));
+#if !defined __RH_CYGLINT__
             set_tty(1);
+#endif
             if ((ch = fgetc(stdin)) != '\n') fputc('\n', stderr);
+#if !defined __RH_CYGLINT__
             set_tty(0);
+#endif
             if (ch != 'n') {
                if ((fp = fopen((*statstr)->home, "a")) == NULL) {
                   fprintf(stderr, "** %s: %s\n", (*statstr)->home, _PFTP_ERROR_ARRAY_);
@@ -177,14 +208,14 @@ int argum(int argc, char **argv, int *portn)
                   *stmp = 0;
                   envtmp = getenv("PFTPCLIENTS");
                   if (argc > (*statstr)->_OPTIONS_ + 1) {
-                     if (!envtmp && *argv[(*statstr)->_OPTIONS_+1] == '-') {
-                        if (!get_var_from_pftprc((FILE *)NULL, "PFTPCLIENTS", stmp, 0)) help();
+                     if (!envtmp && (*argv[(*statstr)->_OPTIONS_+1] == '-' || argv[(*statstr)->_OPTIONS_][i+1])) {
+                        if (!get_var_from_pftprc((FILE *)NULL, "PFTPCLIENTS", stmp, 0)) opt_err_handle(PFTP_OPT_PFTPCLIENTS);
                      }
                      else if (*argv[(*statstr)->_OPTIONS_+1] != '-') clhosts = 1;
                   }
                   else if (argc == (*statstr)->_OPTIONS_ + 1) {
                      if (!envtmp) {
-                        if (!get_var_from_pftprc((FILE *)NULL, "PFTPCLIENTS", stmp, 0)) help();
+                        if (!get_var_from_pftprc((FILE *)NULL, "PFTPCLIENTS", stmp, 0)) opt_err_handle(PFTP_OPT_PFTPCLIENTS);
                      }
                   }
                   if (!*_CLIENTHOSTNAME_ && isatty(2)) {
@@ -281,7 +312,11 @@ int argum(int argc, char **argv, int *portn)
                   (*statstr)->READFROMSTDIN = 1;
                   break;
                case 'e':
+#if !defined __RH_CYGLINT__
                   create_passwd();
+#else
+                  NO_SUCH_OPT();
+#endif
                   break;
                case 'f':
                   if ((cl = argv[(*statstr)->_OPTIONS_][i+1]) == 'S' || cl == 'C') {
@@ -329,9 +364,15 @@ int argum(int argc, char **argv, int *portn)
                         i++;
                      }
                   }
+#if !defined __RH_CYGLINT__
                   if (getpwuid(getuid())) {
                      strcpy((*statstr)->from, (char *)getpwuid(getuid())->pw_name);
                   }
+#else
+                  if (getuid()) {
+                     sprintf((*statstr)->from, "%d", getuid());
+                  }
+#endif
                   else {
                      if (stderr) fprintf(stderr, "** Who are you?!\n");
                      exit(PFTP_UUSER_ERR);
@@ -380,6 +421,7 @@ int argum(int argc, char **argv, int *portn)
                      char *destdir=NULL;
                      MEM_CHECK((destdir = (char *)calloc(LONAME, sizeof(char))));
                      *destdir = '\0';
+#if !defined __RH_CYGLINT__
                      if (*(argv[(*statstr)->_OPTIONS_]+i+2) == '~') {
                         if (!getpwuid(getuid())) {
                            fprintf(stderr, "** getpwuid: %s\n", _PFTP_ERROR_ARRAY_);
@@ -388,6 +430,7 @@ int argum(int argc, char **argv, int *portn)
                         strcpy(destdir, (char *)getpwuid(getuid())->pw_dir);
                         i++;
                      }
+#endif
                      strcat(destdir, (argv[(*statstr)->_OPTIONS_]+i+2));
                      if (chdir(destdir) < 0) {
                         fprintf(stderr, "** %s: %s\n", (argv[(*statstr)->_OPTIONS_]+i+2), _PFTP_ERROR_ARRAY_);
@@ -402,10 +445,7 @@ int argum(int argc, char **argv, int *portn)
                      }
                      _NEXT_ = 0;
                   }
-                  else {
-                     fprintf(stderr, "** You need to specify a destination directory when using `-D'.\n");
-                     exit(PFTP_MCLOPTION_ERR);
-                  }
+                  else opt_err_handle(PFTP_OPT_NO_DIR);
                   break;
                case 'A':
                   if (isdigit(argv[(*statstr)->_OPTIONS_][i+1])) { 
@@ -421,10 +461,7 @@ int argum(int argc, char **argv, int *portn)
                      if (*offset < 0) i += getoffset((argv[(*statstr)->_OPTIONS_]+i+1)) + 1;
                      else help();
                   }
-                  else {
-                     (*statstr)->OVERWRITE = BIT_TWO;
-                     (*statstr)->_SKIP_ = 0;
-                  }
+                  else (*statstr)->OVERWRITE = BIT_TWO;
                   break;
                case 'r':
                   (*statstr)->_RECURS_ = 1;
@@ -433,12 +470,13 @@ int argum(int argc, char **argv, int *portn)
                   (*statstr)->single_connection = 1;
                   break;
                case 'l':
-                  if (!isatty(2)) {
-                     fprintf(stderr, "** Sorry, '-l' needs a tty for output.\n");
+                  if (!isatty(0) || !isatty(1)) {
+                     fprintf(stderr, "\n** Sorry, '-l' needs a tty for input and output!\n\n");
                      exit(PFTP_MCLOPTION_ERR);
                   }
-                  else if (_ISDIGIT_ == 1) help();
+                  else if (_ISDIGIT_ == 1) opt_err_handle(PFTP_OPT_LIST_OR_NUM);
                   else {
+                     if ((*statstr)->_STANDARD_INPUT_ || !isatty(0)) opt_err_handle(PFTP_OPT_LIST);
                      lhost(portn, 0);
                      _ISDIGIT_ = 2;
                   }
@@ -463,27 +501,27 @@ int argum(int argc, char **argv, int *portn)
                case 's':
                   (*statstr)->show_bytes = BIT_ONE;
                   (*statstr)->_SKIP_ = BIT_ONE;
-                  (*statstr)->OVERWRITE = 0;
                   (*statstr)->show_percent = 1;
                   break;
                case 'q':
                   slfp = NULL;
                   break;
-               case 'Q':
-                  (*statstr)->QT_FRONTEND = 1;
-                  (*statstr)->READFROMSTDIN = 1;
+               case 'K':
+                  (*statstr)->KDE_FRONTEND = 1;
                   break;
-#ifdef HAVE_INET6
                case 'P':
+#if defined SO_PRIORITY
                   /*
-                   * Set datagram priority for IPv6.
+                   * Set packet priority.
                    */
                   if (isdigit(argv[(*statstr)->_OPTIONS_][i+1])) {
                      i = extract_val_i((unsigned int *)&((*statstr)->priority), argv[(*statstr)->_OPTIONS_], i);
                   }
                   else (*statstr)->priority = PFTP_DEFAULT_PRIORITY;
-                  break;
+#else
+                  fprintf(stderr, "** Setting packet priority is not supported on this system.\n");
 #endif
+                  break;
                case 'S':
                   /*
                    * Sets the period of dropping datagrams.
@@ -505,6 +543,9 @@ int argum(int argc, char **argv, int *portn)
                   }
                   else (*statstr)->testbsize = 1L;
                   break;
+               case 'G':
+                  GIGABIT_ETHERNET = 1;
+                  break;
                case 'F':
                   (*statstr)->autobitrate = 1;
                   break;
@@ -575,8 +616,8 @@ int argum(int argc, char **argv, int *portn)
                   break;
                default:
                   if (_ISDIGIT_ > 1 || !isdigit(c)) {
-                     fprintf(stderr, "**\n** unknown option `%c'\n**\n", c);
-                     help();
+                     fprintf(stderr, "\n** Option `-%c' is not recognized by pftp!\n\n", c);
+                     opt_err_handle(PFTP_OPT_UNKNOWN_OPT);
                   }
                   else if (!_ISDIGIT_) {
                      (*statstr)->host_number = atoi((argv[(*statstr)->_OPTIONS_]+i));
@@ -652,10 +693,31 @@ int argum(int argc, char **argv, int *portn)
       exit(PFTP_NO_ERR);
    }
 
+   /*
+    * Set special values for Gigabit Ethernet links.
+    * `-B' and `-bNUM' are ignored.
+    */
+   if (GIGABIT_ETHERNET) {
+      if (!(*statstr)->use_udp) {
+         size_t gigatmp=0;
+         (*statstr)->bsize = DEFAULT_GIGABIT_NETSZ;
+         (*statstr)->_STDIN_BUFSIZ_ = DEFAULT_GIGABIT_STRSZ;
+         MEM_CHECK((gtmp = (char *)calloc(SONAME, sizeof(char))));
+         if (get_var_from_pftprc((FILE *)NULL, "PFTPGIGASTR", gtmp, 0)) {
+            if ((gigatmp = atol(gtmp))) (*statstr)->bsize = gigatmp;
+         }
+         if (get_var_from_pftprc((FILE *)NULL, "PFTPGIGANET", gtmp, 0)) {
+            if ((gigatmp = atol(gtmp))) (*statstr)->_STDIN_BUFSIZ_ = gigatmp;
+         }
+         free(gtmp);
+      }
+      else opt_err_handle(PFTP_OPT_GIGA_UDP);
+   }
+
    /*
     * Set the net buffer size.
     */ 
-   if (set_netbuffer) {
+   else if (set_netbuffer) {
       MEM_CHECK((gtmp = (char *)calloc(SONAME, sizeof(char))));
       if (!get_var_from_pftprc((FILE *)NULL, "PFTPNETBUF", gtmp, 0)) {
          fprintf(stderr, "** Please set the variable PFTPNETBUF!\n");
@@ -681,11 +743,14 @@ int argum(int argc, char **argv, int *portn)
    /*
     * If not already set adjust buffer to bandwidth.
     */
-   if ((*statstr)->_BANDWIDTH_  && ((*statstr)->_BANDWIDTH_ < BUFSIZE) && !(*statstr)->_STDIN_BUFSIZ_ && !(*statstr)->_STANDARD_INPUT_) {
+   if ((*statstr)->_BANDWIDTH_  && ((*statstr)->_BANDWIDTH_ < BUFSIZE) \
+      && !(*statstr)->_STDIN_BUFSIZ_ && !(*statstr)->_STANDARD_INPUT_) {
       (*statstr)->_STDIN_BUFSIZ_ = (*statstr)->_BANDWIDTH_;
    }
-
-   if (!(*statstr)->_STDIN_BUFSIZ_ && !(*statstr)->_STANDARD_INPUT_ && !(*statstr)->FORCE_STDOUT) (*statstr)->_STDIN_BUFSIZ_ = DEFAULT_STDOUT_BUFSIZ;
+   else if (!(*statstr)->_STDIN_BUFSIZ_ && !(*statstr)->_STANDARD_INPUT_ \
+      && !(*statstr)->FORCE_STDOUT) {
+      (*statstr)->_STDIN_BUFSIZ_ = DEFAULT_STDOUT_BUFSIZ;
+   }
 
    /*
     * Indentify and check.
@@ -713,44 +778,59 @@ int argum(int argc, char **argv, int *portn)
       exit(PFTP_MCLOPTION_ERR);
    }
    if ((*statstr)->testbsize && (*statstr)->use_udp) (*statstr)->_STANDARD_INPUT_ = BIT_ONE;
-   if(((*statstr)->OVERWRITE && (*statstr)->_SKIP_) || ((*statstr)->OVERWRITE && _ISDIGIT_)) help();
    /*
     * If the program is running as the server return 1 else 
     * if the program is running as the client return 0.
-    * Print help a message if an error occurs.
+    * Print a help message if an error occurs.
     */
+   if ((*statstr)->use_udp && (*statstr)->_PFTP_DAEMON_) opt_err_handle(PFTP_OPT_DAEMON_UDP);
+   else if ((*statstr)->use_udp && (*statstr)->usefilter) opt_err_handle(PFTP_OPT_FILTER_UDP);
    else if ((((argc == 2) && (!isalpha(argv[1][0]))) \
-        || ((*statstr)->_OPTIONS_ == argc)) \
-        && (!_ISDIGIT_) \
-        && (!(*statstr)->_BANDWIDTH_) && (*offset < 0) \
-        && !((*statstr)->_PFTP_DAEMON_ && (*statstr)->use_udp) \
-        && !((*statstr)->READFROMSTDIN) \
-        && !(*statstr)->SENDMAKELINKS \
-        && !*((*statstr)->from))
-        /*
-         * Program is running as the server.
-         */
-        return 1;
-   else if (((*statstr)->OVERWRITE || (*statstr)->_SKIP_ || *_CLIENTHOSTNAME_) \
-        || (_ISDIGIT_ > 1 && (*statstr)->_STANDARD_INPUT_) \
-        || ((*statstr)->_RECURS_ && (*statstr)->_STANDARD_INPUT_ && !(*statstr)->testbsize) \
-        || ((*statstr)->_RECURS_ && !(*statstr)->testbsize && !_ISDIGIT_ && !(*statstr)->READFROMSTDIN \
-        && (*statstr)->_OPTIONS_ == argc - 1) \
-        || ((*statstr)->_STANDARD_INPUT_ && (*statstr)->READFROMSTDIN && ((*statstr)->_OPTIONS_ + 1 < argc)) \
-        || (!(*statstr)->_STANDARD_INPUT_ && !(*statstr)->READFROMSTDIN && (argv[argc-1][0] == '-' && !_MESSAGE_ONLY_) \
-        && !((*statstr)->use_udp && (*statstr)->testbsize)) \
-        || ((*statstr)->_STANDARD_INPUT_ && argv[argc-1][0] != '-'  && _ISDIGIT_) \
-        || ((*statstr)->exit_afc) \
-        || ((*statstr)->rename && !(*statstr)->testbsize) \
-        || ((*statstr)->FORCE_STDOUT) \
-        || (!(*statstr)->_STANDARD_INPUT_ && !(*statstr)->READFROMSTDIN && argc < 3 - _MESSAGE_ONLY_) \
-        || ((*statstr)->use_udp && (*statstr)->_PFTP_DAEMON_))
-           help();
-
-   if (((*statstr)->READFROMSTDIN || (*statstr)->_STANDARD_INPUT_) && (*statstr)->_OPTIONS_ == argc) {
-      fprintf(stderr, "** No host name specified!\n");
-      exit(PFTP_HOSTNAME_ERR);
+           || ((*statstr)->_OPTIONS_ == argc)) \
+           && !_ISDIGIT_ \
+           && (!(*statstr)->_BANDWIDTH_) && (*offset < 0) \
+           && !(*statstr)->READFROMSTDIN \
+           && !(*statstr)->SENDMAKELINKS \
+           && !_MESSAGE_ONLY_ \
+           && !*((*statstr)->from)) {
+      if(((*statstr)->OVERWRITE && (*statstr)->_SKIP_)) opt_err_handle(PFTP_OPT_OVERWRT_SKIP);
+#if defined __RH_CYGLINT__
+      if ((*statstr)->_PFTP_DAEMON_) {
+         fprintf(stderr, "** The daemon does not run on this operating system!\n");
+         exit(1);
+      }
+#endif
+      /*
+       * Program is running as the server.
+       */
+      return 1;
    }
+   else if ((*statstr)->OVERWRITE || (*statstr)->_SKIP_ \
+           || *_CLIENTHOSTNAME_ || (*statstr)->FORCE_STDOUT \
+           || (*statstr)->exit_afc) opt_err_handle(PFTP_OPT_USELESS);
+   else if (_ISDIGIT_ > 1 && (*statstr)->_STANDARD_INPUT_) opt_err_handle(PFTP_OPT_LIST);
+   else if ((*statstr)->_RECURS_ && (*statstr)->_STANDARD_INPUT_ \
+           && !(*statstr)->testbsize) opt_err_handle(PFTP_OPT_RECURS_STDIN);
+   else if (!_ISDIGIT_ && (((*statstr)->_RECURS_ && !(*statstr)->testbsize \
+           && !(*statstr)->READFROMSTDIN && (*statstr)->_OPTIONS_ == argc - 1) \
+           || (!(*statstr)->_STANDARD_INPUT_ && !(*statstr)->READFROMSTDIN \
+           && ((argv[argc-1][0] == '-' || argv[argc-2][0] == '-') && !_MESSAGE_ONLY_) \
+           && !((*statstr)->use_udp && (*statstr)->testbsize)) \
+           || (((*statstr)->READFROMSTDIN || (*statstr)->_STANDARD_INPUT_ \
+           || _MESSAGE_ONLY_) && (*statstr)->_OPTIONS_ == argc) \
+           || (!(*statstr)->_STANDARD_INPUT_ && !(*statstr)->READFROMSTDIN \
+           && argc < 3 - _MESSAGE_ONLY_))) opt_err_handle(PFTP_OPT_NO_HOST);
+   else if ((*statstr)->_STANDARD_INPUT_ && argv[argc-1][0] != '-'  \
+           && (_ISDIGIT_ || argv[argc-2][0] != '-')) opt_err_handle(PFTP_OPT_STDIN_FILES);
+   else if ((*statstr)->rename && !(*statstr)->testbsize) opt_err_handle(PFTP_OPT_RENAME);
+   else if ((*statstr)->use_udp && (*statstr)->usefilter) opt_err_handle(PFTP_OPT_FILTER_UDP);
+   else if (_ISDIGIT_ && !_MESSAGE_ONLY_ && (*statstr)->_OPTIONS_ == argc - 1 \
+           && !(*statstr)->READFROMSTDIN && !(*statstr)->_STANDARD_INPUT_) {
+      opt_err_handle(PFTP_OPT_NO_FILES);
+   }
+#if !defined SO_PRIORITY
+   else if ((*statstr)->priority) opt_err_handle(PFTP_OPT_PRIORITY);
+#endif
 
    /*
     * Program is running as the client.
