@@ -1,7 +1,7 @@
 /*
  * pftp -- sends files from host to host through free choosable ports
  *
- * Copyright (C) 1996-1999 Ben Schluricke
+ * Copyright (C) 1996-2000 Ben Schluricke
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,16 +30,29 @@
 #endif
 #include "main.h"
 
+#if !defined __RH_CYGLINT__
 extern void set_tty(int);
 extern void display_pa(char **, char *, char *, int);
 extern char *headline;
+#else
+char *headline;
+#endif
 char tmpline[LONAME];
 
-#define HELPLINES 101
+#define HELPLINES 102
 #define PFM_HELPLINES 38
 #define PAGER_HELPLINES 16
+#ifdef SO_PRIORITY
+#define MAX_OPT_ERRORS 14
+#else
+#define MAX_OPT_ERRORS 15
+#endif
 
-void help() {
+
+void help()
+{
+   int j=0;
+   char *ctmp=NULL;
    char *helpstr[HELPLINES+1] = {
       " Server: pftp [port] [OPTIONS]                                   \n",
       "                                                                 \n",
@@ -65,6 +78,7 @@ void help() {
       "    -D=DIR    Change to directory DIR before receiving data.     \n",
       "    -f[NUM]   Use filter NUM specified in the variable           \n",
       "              PFTPSFILTER.  NUM defaults to 1.                   \n",
+      "    -G        Set optimized values for Gigabit Ethernet links.   \n",
       "    -i        Start pftp from `/etc/inetd.conf'.  The number of  \n",
       "              the filter to be used is specified by the client.  \n",
       "              The filter is read from user's pftp resource file. \n",
@@ -88,7 +102,11 @@ void help() {
       "    -r        Receive directories recursively.                   \n",
       "    -s        TCP: Skip all existing files.  UDP: Display bytes. \n",
       "    -T        Net performance test (requires `-b' or `-u').      \n",
+#if defined HAVE_INET6
+      "    -u[NUM]   Use UDP.  NUM is the string size (default: 1024).  \n",
+#else
       "    -u[NUM]   Use UDP.  NUM is the string size (default: 1472).  \n",
+#endif
       "    -v        Be a little more verbose.                          \n",
       "    -x        Exit after first connection.  Implies `-o'.        \n",
       "                                                                 \n",
@@ -107,17 +125,18 @@ void help() {
       "              value specified by NUM, or if omitted the value of \n",
       "              the environment variable PFTPNETBUF is taken.      \n",
       "    -BB       Same as `-B65536'.                                 \n",
-      "    -d[=DIR]  Send data to the daemon.  The files are uploaded   \n",
-      "              in the directory DIR on the remote host.           \n",
+      "    -d[=DIR]  Same as `-i' except that you can choose a port.    \n",
       "    -e        Create a normal password with crypt().             \n",
       "    -f[NUM]   Use filter NUM specified in variable PFTPCFILTER.  \n",
       "              NUM defaults to 1.                                 \n",
       "    -F        Determine and set bandwidth of MPEG AUDIO streams. \n",
-      "    -i[=DIR]  Send data to the inetd server.  The files are      \n",
+      "    -G        Set optimized values for Gigabit Ethernet links.   \n",
+      "    -i[=DIR]  Send data to the daemon on port 662.  The files are\n",
       "              uploaded in the directory DIR on the remote host.  \n",
       "    -I        Send last modification time, uid, gid from files   \n",
       "              and diretories, and diretory permission bits.      \n",
       "    -j        Read file and directory names from standard input. \n",
+      "    -K        Used by pftp's KDE front end, kpftp.               \n",
       "    -l        Select hostname from list and send the files       \n",
       "              specified on command line.                         \n",
       "    -L        Do not follow symbolic links. Instead send a script\n",
@@ -127,6 +146,11 @@ void help() {
       "    -mi IADDR Use interface IADDR for multicasted datagrams.     \n",
       "    -ml       Multicasted datagrams will be looped back.         \n",
       "    -mtNUM    Time to live (IPv4) resp. hop limit (IPv6).        \n",
+#if defined HAVE_INET6
+      "              This binary supports IPv4 and IPv6.                \n",
+#else
+      "              This binary supports IPv4 only.                    \n",
+#endif
       "    -M[=FILE] Add message file FILE or if FILE is not given write\n",
       "              the file interactively. Implies `-n'.              \n",
       "    -N        Peek for new data in the upload directory.         \n",
@@ -135,32 +159,30 @@ void help() {
       "    -n[=SUBJ] Send data [of subject SUBJ] to a friend.           \n",
       "    -NUM      NUM is the number of the host in your pftp resource\n",
       "              file.  Try `pftp -l' to see the numbers.           \n",
-      "    -P[NUM]   Set datagram priority to NUM (for IPv6).           \n",
-#if defined HAVE_INET6
-      "              This binary supports IPv4 and IPv6.                \n",
-#else
-      "              This binary supports IPv4 only.                    \n",
-#endif
+      "    -P[NUM]   Set datagram priority, if supported by your system.\n",
       "    -q        Run silently.                                      \n",
-      "    -Q        Used by the pftp front end, qpftp.                 \n",
       "    -r        Send directories recursively.                      \n",
       "    -SNUM     UDP: Drop every NUMth datagram.                    \n",
       "    -T[NUM]   Net performance test with NUM bytes to send.       \n",
+#if defined HAVE_INET6
+      "    -u[NUM]   Use UDP.  NUM is the string size (default: 1024).  \n",
+#else
       "    -u[NUM]   Use UDP.  NUM is the string size (default: 1472).  \n",
+#endif
       "    -v        Be a little more verbose.                          \n",
       "    -w[NUM]   Set the bandwidth in bytes to NUM, or if omitted to\n",
       "              the value of the environment variable PFTPBANDWID. \n",
       "    -W        Display warranty.                                  \n"
    };
    
+   MEM_CHECK((headline = (char *)calloc(SONAME, sizeof(char))));
+   sprintf(headline, "*** Port-FTP Version " VERSION " Copyright (C) 1996-2000 Ben Schluricke ***"); 
+   for (ctmp=headline; *ctmp; ctmp++);
+   for (; ctmp - headline < _WINCOLS_; ctmp++) *ctmp = '*';
+   *(ctmp-1) = ' ';
+   *ctmp = '\0';
+#if !defined __RH_CYGLINT__
    if (isatty(0) && isatty(1)) {
-      char *ctmp=NULL;
-      MEM_CHECK((headline = (char *)calloc(SONAME, sizeof(char))));
-      sprintf(headline, "*** Port-FTP Version %s Copyright (C) 1996-1999 Ben Schluricke ***", VERSION); 
-      for (ctmp=headline; *ctmp; ctmp++);
-      for (; ctmp - headline < _WINCOLS_; ctmp++) *ctmp = '*';
-      *(ctmp-1) = ' ';
-      *ctmp = '\0';
       fprintf(stdout, "[?25l[H[J");
       set_tty(2);
       memset(tmpline, ' ', _WINCOLS_ < LONAME ? _WINCOLS_: LONAME);
@@ -168,19 +190,82 @@ void help() {
       display_pa(helpstr, headline, tmpline, HELPLINES);  
       set_tty(0);
       fprintf(stderr, "[?25h\r                                                                       \r");
-      free(headline);
    }
    else {
-      int j=0;
-      fprintf(stdout, "\n*** Port-FTP Version %s ", VERSION); 
-      fprintf(stdout, "Copyright (C) 1996-1999 Ben Schluricke ***\n\n");
+#endif
+      fprintf(stdout, "\n%s\n\n", headline);
       for (j=0; j <= HELPLINES; j++) fputs(*(helpstr+j), stdout);
+#if !defined __RH_CYGLINT__
    }
+#endif
+   free(headline);
    exit(PFTP_NO_ERR);
 }
 
 
-void mhelp() {
+void opt_err_handle(int opt_err)
+{
+   char *opt_err_str[MAX_OPT_ERRORS] = {
+      "It makes no sense to use `-a', `-c', `-p', `-s', or `-x' with the client",
+      "`-l' cannot be used with `-[b[NUM]]' or `-j'.\n** Please, use option `-NUM' instead",
+      "`-r' cannot be used in standard input mode with the client",
+      "No host name specified for the client",
+      "You can either send data from standard input (`-[b[NUM]]')\n** or send files, but not both at once",
+      "`-R' is used with the server, or with the client in combination with `-T'",
+      "Cannot send UDP datagrams (`-u') to the daemon",
+      "You can either overwrite (`-a') or skip (`-s') existing files",
+      "Variable `PFTPCLIENTS' is unset or empty",
+      "No files given on command line",
+      "You can either use `-l' or `-NUM'",
+      "`-G' is *not* meant for UDP based connections (`-u')",
+      "You cannot use filters (`-f') for UDP based connections (`-u')",
+      "You need to specify a destination directory when using `-D'",
+#if !defined SO_PRIORITY
+      "`-P' is only supported on the Linux Operating System"
+#endif
+   };
+   char *progstr[3] = {
+      "man",
+      "pftp",
+   };
+   char c;
+
+   /*
+    * Output error message.
+    */
+   if (opt_err >= MAX_OPT_ERRORS) {
+      fprintf(stderr, "** Unknown error!\n");
+   }
+   else if (opt_err >= 0) fprintf(stderr, "\n** %s!\n\n", *(opt_err_str+opt_err));
+
+#if !defined __RH_CYGLINT__
+   if (isatty(0) && isatty(1)) {
+      fprintf(stderr, "Quit (q), help list (h), manual page (m)? ");
+      set_tty(2);
+      c = fgetc(stdin);
+      set_tty(0);
+      switch(c) {
+         case 'h':
+            help();
+            break;
+         case 'm':
+            fputc('\n', stderr); 
+            execvp(*progstr, progstr);
+            fprintf(stderr, "** %s: %s\n", *progstr, _PFTP_ERROR_ARRAY_);
+            exit(PFTP_PROG_ERR);
+         case 'q':
+         default:
+            fprintf(stderr, "quit\n");
+      }
+   }
+#endif
+   exit(PFTP_MCLOPTION_ERR);
+}
+
+
+#if !defined __RH_CYGLINT__
+void mhelp()
+{
    char *helpstr[PFM_HELPLINES+1] = {
       "   List of commands for the file and directory manager               \n",
       "                                                                     \n",
@@ -230,7 +315,8 @@ void mhelp() {
 }
 
 
-void pager_help() {
+void pager_help()
+{
    char *helpstr[PAGER_HELPLINES+1] = {
       "   List of commands for the internel pager                                   \n",
       "                                                                     \n",
@@ -262,3 +348,57 @@ void pager_help() {
    fprintf(stdout, "[H[J");
    used = 0;
 }
+#endif
+
+#define WHELPLINES 14
+
+void warranty(void) {
+   int j=0;
+   char *firstline=NULL, *blankline=NULL, *ctmp=NULL;
+   char *warrantystr[WHELPLINES+1] = {
+      "Copyright (C) 1996-2000 Ben Schluricke                                 \n",
+      "                                                                       \n",
+      "This program is free software; you can redistribute it and/or modify   \n",
+      "it under the terms of the GNU General Public License as published by   \n",
+      "the Free Software Foundation; either version 2 of the License, or (at  \n",
+      "your option) any later version.                                        \n",
+      "                                                                       \n",
+      "This program is distributed in the hope that it will be useful, but    \n",
+      "WITHOUT ANY WARRANTY; without even the emplied warranty of MERCHANT-   \n",
+      "ABILITY OF FITNESS FOR A PARTICULAR PURPOSE. See the GNU General       \n",
+      "Public License for more details.                                       \n",
+      "                                                                       \n",
+      "You should have received a copy of the GNU General Public License      \n",
+      "along with this program; if not, write to the Free Software Foundation,\n",
+      "Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                          \n"
+   };
+
+   MEM_CHECK((firstline = (char *)calloc(LONAME, sizeof(char))));
+   sprintf(firstline, "****** Port-FTP Version " VERSION " Copyright (C) 1996-2000 Ben Schluricke "); 
+   for (ctmp=firstline; *ctmp; ctmp++);
+   for (; ctmp - firstline < _WINCOLS_; ctmp++) *ctmp = '*';
+   *(ctmp-1) = ' ';
+   *ctmp = '\0';
+#if !defined __RH_CYGLINT__
+   if (isatty(0) && isatty(1)) {
+      MEM_CHECK((blankline = (char *)calloc(SONAME, sizeof(char))));
+      for (ctmp=blankline; ctmp - blankline < _WINCOLS_; ctmp++) *ctmp = ' ';
+      *ctmp='\r';
+      *++ctmp='\0';
+      fprintf(stdout, "[H[J");
+      ctmp = headline;
+      headline = firstline;
+      display_pa(warrantystr, firstline, blankline, WHELPLINES);  
+      headline = ctmp;
+      free(blankline);
+   }
+   else {
+#endif
+      fprintf(stdout, "\n%s\n\n", firstline);
+      for (j=0; j <= WHELPLINES; j++) fputs(*(warrantystr+j), stdout);
+      fputc('\n', stdout);
+#if !defined __RH_CYGLINT__
+   }
+#endif
+   free(firstline);
+}
