@@ -1,7 +1,7 @@
 /*
  * pftp -- sends files from host to host through free choosable ports
  *
- * Copyright (C) 1996-1999 Ben Schluricke
+ * Copyright (C) 1996-2000 Ben Schluricke
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  *
  *
  */
-#ifdef USE_POSIX_THREAD
+#if defined USE_POSIX_THREAD
 #define _REENTRANT
 #include <pthread.h>
 #endif
@@ -122,7 +122,7 @@ void pmanager(short PFTP_PEEK)
 
    MEM_CHECK((cdir = (char *)calloc(LONAME, sizeof(char))));
    MEM_CHECK((headline = (char *)calloc(LONAME, sizeof(char))));
-   sprintf(headline, "****** Port-FTP Version %s ****** file and directory manager ", VERSION);
+   sprintf(headline, "****** Port-FTP Version " VERSION " ****** file and directory manager ");
    for (ctmp=headline; *ctmp; ctmp++);
    for (; ctmp - headline < _WINCOLS_; ctmp++) *ctmp = '*';
    *(ctmp-1) = ' ';
