@@ -1,7 +1,7 @@
 /*
  * pftp -- sends files from host to host through free choosable ports
  *
- * Copyright (C) 1996-1999 Ben Schluricke
+ * Copyright (C) 1996-2000 Ben Schluricke
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,19 +37,25 @@
 #include <sys/time.h>
 #include "main.h"
 
+#if !defined __RH_CYGLINT__
 extern void set_tty(int);
+#endif
 extern short get_var_from_pftprc(FILE *, const char *, char *, int);
 extern void import(unsigned long, double *, int);
 extern void timing(int, double *, char *);
 extern char *time_string(void);
+#if !defined __RH_CYGLINT__
 extern short check_pass(int);
 extern char *tabdir(char *);
+#endif
 extern void receive_udps(int strnum, double *);
 extern void find_free_name(char *, char *);
 extern int get_file_status(int, pftp_file_status *);
 extern int skip_upper_dirs(int, int, int);
 extern void send_oob_error(int, char *);
+#if !defined __RH_CYGLINT__
 extern int check_dir_size(int, double *);
+#endif
 #if defined Linux && defined USE_POSIX_THREAD
 extern void signal_handling(int);
 #endif
@@ -57,6 +63,12 @@ extern void signal_handling(int);
 extern void test_the_net_read(int, double *);
 static double sum_bytes[MAXCLIENTS+1];
 
+#if defined __RH_CYGLINT__
+#define PFTP__OPEN_OPTION__ O_CREAT|O_WRONLY|O_BINARY|appending
+#else
+#define PFTP__OPEN_OPTION__ O_CREAT|O_WRONLY|appending
+#endif
+
 void receive_data(void *structure_number)
 {
    char fstr[HUNAME], ch[1];
@@ -80,6 +92,7 @@ void receive_data(void *structure_number)
    if (BUFFER_SIZE > BIG_BUFSIZE) BUFFER_SIZE = BIG_BUFSIZE;
 #endif
 
+#if !defined __RH_CYGLINT__
    if ((*statstr)->_PFTP_DAEMON_) {
       /*
        * Check password if server is running
@@ -103,6 +116,9 @@ void receive_data(void *structure_number)
       }
    }
    else if (!(*(statstr+strnum))->accept_file_info) {
+#else
+   if (!(*(statstr+strnum))->accept_file_info) {
+#endif
       /*
        * Look if PFTPSINFO is set.
        */
@@ -178,9 +194,11 @@ void receive_data(void *structure_number)
              */
             if (cc == 3) return;
             filesize = pfs.size;
+#if !defined __RH_CYGLINT__
             if (*((*(statstr+strnum))->from) && (*(statstr+strnum))->mul) {
                if (!check_dir_size(strnum, &filesize)) return;
             }
+#endif
             mode = pfs.mode;
             umask(~mode);
             umask(~smode);
@@ -275,7 +293,9 @@ void receive_data(void *structure_number)
 #endif
                }
 #else
+#if !defined __RH_CYGLINT__
                set_tty(1);
+#endif
 #endif
                hor=fgetc(stdin);
 #ifdef Linux
@@ -285,7 +305,9 @@ void receive_data(void *structure_number)
                }
 #endif
 #else
+#if !defined __RH_CYGLINT__
                set_tty(0);
+#endif
 #endif
                fprintf(stderr, "\r                                                               \r");
                if(hor=='Y' || hor=='y' || hor=='a') {
@@ -340,6 +362,7 @@ void receive_data(void *structure_number)
                   char *str=NULL;
                   char *stmp=NULL;
                   MEM_CHECK((str = (char *)calloc(SONAME, sizeof(char))));
+#if !defined __RH_CYGLINT__
 #ifdef NEXTSTEP
                   getwd(str);
 #else
@@ -351,6 +374,12 @@ void receive_data(void *structure_number)
                   stmp = tabdir(str);
 #ifndef Linux
                   set_tty(0);
+#endif
+#else
+                  if (!(stmp = fgets(str, SONAME, stdin))) {
+                     send_oob_error(strnum, (*(statstr+strnum))->cwd_last_ch);
+                     return;
+                  }
 #endif
                   fprintf(stderr, "\n");
                   if (!stat(stmp, &buf) && (buf.st_mode & S_IFDIR)) {
@@ -359,7 +388,7 @@ void receive_data(void *structure_number)
                   }
                   if (stmp) strcpy((*(statstr+strnum))->cwd, stmp);
                   free(str);
-               }   
+               }
 #if defined Linux
                if ((*statstr)->ttys) {
 #ifdef USE_POSIX_THREAD
@@ -390,7 +419,7 @@ void receive_data(void *structure_number)
                /*
                 * Open file on local host for writing.
                 */
-               if ((ft = open((*(statstr+strnum))->cwd, O_CREAT|O_WRONLY|appending, mode)) < 0) {
+               if ((ft = open((*(statstr+strnum))->cwd, PFTP__OPEN_OPTION__, mode)) < 0) {
                   send_oob_error(strnum, (*(statstr+strnum))->cwd_last_ch);
                   return;
                }
@@ -414,9 +443,9 @@ void receive_data(void *structure_number)
       else strcpy((*(statstr+strnum))->cwd_last_ch, "Datagrams");
 
       if ((*(statstr+strnum))->_SKIP_ > 1) {
-         register int rr = (unsigned int)BUFFER_SIZE < filesize ? BUFFER_SIZE : filesize;
+         register size_t rr = (size_t)((size_t)BUFFER_SIZE < (size_t)filesize) ? (size_t)BUFFER_SIZE : (size_t)filesize;
          while (ci < filesize && (cc = read(fd, bstr, rr)) > 0) {
-            if ((ci+=(double)cc) > (filesize - (double)rr)) rr = (int)(filesize - ci);
+            if ((ci+=(double)cc) > (filesize - (double)rr)) rr = (size_t)(filesize - ci);
          }
       }
       /*
@@ -424,10 +453,10 @@ void receive_data(void *structure_number)
        * local file respectively standard out.
        */
       else if (filesize) {
-         register int rr = (unsigned int)((double)BUFFER_SIZE < filesize) ? BUFFER_SIZE : filesize;
+         register size_t rr = (size_t)((size_t)BUFFER_SIZE < (size_t)filesize) ? (size_t)BUFFER_SIZE : (size_t)filesize;
          double firr = filesize - (double)rr;
          while (ci < filesize && (cc = read(fd, bstr, rr)) > 0) {
-            if ((ci+=(double)cc) > firr) rr = (int)(filesize - ci);
+            if ((ci+=(double)cc) > firr) rr = (size_t)(filesize - ci);
             for (check=0, strptr=bstr; cc; cc-=check, strptr += check) {
                if ((check = write(ft, strptr, cc)) < 0) {
                   send_oob_error(strnum, (*(statstr+strnum))->cwd_last_ch);
@@ -435,6 +464,7 @@ void receive_data(void *structure_number)
                }
             }
          }
+#if !defined __RH_CYGLINT__
          if (pfs.info) {
             if (!getuid() && fchown(ft, pfs.uid, pfs.gid)) {
                send_oob_error(strnum, (*(statstr+strnum))->cwd_last_ch);
@@ -442,12 +472,15 @@ void receive_data(void *structure_number)
                return;
             }
          }
+#endif
       }
       else {
          if (!(*statstr)->use_udp) {
+#if !defined __RH_CYGLINT__
             if (!*((*(statstr+strnum))->from)) {
+#endif
                if (!(*statstr)->testbsize) {
-                  register int rr = BUFFER_SIZE;
+                  register size_t rr = BUFFER_SIZE;
                   if (!(*statstr)->_STDIN_BUFSIZ_) rr = 1;
                   for (ci=0, cc=0; (cc = read(fd, bstr, rr)) > 0;) {
                      ci+=(double)cc;
@@ -460,9 +493,10 @@ void receive_data(void *structure_number)
                   }
                }
                else test_the_net_read(fd, &ci);
+#if !defined __RH_CYGLINT__
             }
             else {
-               register int rr = BUFFER_SIZE;
+               register size_t rr = BUFFER_SIZE;
                for (ci=0.0, cc=0; (cc = read(fd, bstr, rr)) > 0;) {
                   filesize = (double)cc;
                   if ((*(statstr+strnum))->mul && !check_dir_size(strnum, &filesize)) {
@@ -479,6 +513,7 @@ void receive_data(void *structure_number)
                   }
                }
             }
+#endif
          }
          /*
           * Handle UDP datagrams.
