@@ -1,7 +1,7 @@
 /*
  * pftp -- sends files from host to host through free choosable ports
  *
- * Copyright (C) 1996-1999 Ben Schluricke
+ * Copyright (C) 1996-2000 Ben Schluricke
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,14 +47,13 @@
 #if defined SunOS_SOLARIS || defined unicos
 #include <fcntl.h>
 #endif
-#include <setjmp.h>
 #include "main.h"
-#ifdef unicos
+#if defined unicos || defined HP_UX
 #define PFTP_THREAD_CAST (void *(*)(void *))
 #else
 #define PFTP_THREAD_CAST (void *)
 #endif
-#if defined __GNUC__ && defined __GNUC_MINOR__ && __GNUC__ > 1 && __GNUC_MINOR__ > 8
+#if defined __GNUC__ && defined __GNUC_MINOR__ && __GNUC__ > 1 && __GNUC_MINOR__ > 8 && !defined __RH_CYGLINT__
 typedef socklen_t UNSIGNED_ARG_INT;
 #else
 typedef int UNSIGNED_ARG_INT;
@@ -69,7 +68,9 @@ extern void signal_handling(int);
 extern int init_struct(void);
 extern short get_var_from_pftprc(FILE *, const char *, char *, int);
 extern short check_login(int);
+#if !defined __RH_CYGLINT__
 extern int init_user_id(int);
+#endif
 extern char *time_string(void);
 extern int check_accepted_hosts(char **, int);
 extern int search_hl_entry(int);
@@ -359,6 +360,7 @@ void login_handle(void *structure_number)
    struct sockaddr_in6 *fsin = &(*(statstr+strnum))->sin;
    int fsinlen=(*(statstr+strnum))->sinlen;
    int ret=0;
+   char cptmp[IPv6_BIN_ADDR_LENGTH];
 #else
    struct hostent *hp;
    struct sockaddr_in *fsin = &(*(statstr+strnum))->sin;
@@ -384,28 +386,13 @@ void login_handle(void *structure_number)
 #endif
             return;
          }
-         else strcpy((*(statstr+strnum))->REMOTEIP, (*(statstr+strnum))->REMOTEHOSTNAME);
-      }
-      else if (!inet_ntop(fsin->sin6_family, fsin->sin6_addr.s6_addr, \
-              (*(statstr+strnum))->REMOTEIP, INET6_ADDRSTRLEN)) {
-         if (slfp) {
-            fprintf(slfp, "%s Cannot convert binary address string\n", \
-                   (*statstr)->_PFTP_DAEMON_ ? time_string(): "**");
-         }
-         close((*(statstr+strnum))->ns);
-#ifdef USE_POSIX_THREAD
-         (*(statstr+strnum))->free = 1;
-#endif
-         return;
       }
 #else
       if ((hp = gethostbyaddr((char *)&(fsin->sin_addr), sizeof(fsin->sin_addr), AF_INET)) == 0) {
          strcpy((*(statstr+strnum))->REMOTEHOSTNAME, inet_ntoa(fsin->sin_addr));
-         strcpy((*(statstr+strnum))->REMOTEIP, (*(statstr+strnum))->REMOTEHOSTNAME);
       }
       else {
          strcpy((*(statstr+strnum))->REMOTEHOSTNAME, hp->h_name);
-         strcpy((*(statstr+strnum))->REMOTEIP, inet_ntoa(fsin->sin_addr));
       }
 #endif
 
@@ -418,10 +405,18 @@ void login_handle(void *structure_number)
           */
          if (!check_accepted_hosts(_CLIENTHOSTNAME_, strnum)) {
             if (slfp) {
+#ifdef HAVE_INET6
                fprintf(slfp, "%s Connection to %s (%s) refused\n", \
                (*statstr)->_PFTP_DAEMON_ ? time_string(): "**", \
                (*(statstr+strnum))->REMOTEHOSTNAME, \
-               (*(statstr+strnum))->REMOTEIP);
+               inet_ntop(fsin->sin6_family, fsin->sin6_addr.s6_addr, cptmp, IPv6_BIN_ADDR_LENGTH) ? \
+               cptmp: (*(statstr+strnum))->REMOTEHOSTNAME);
+#else
+               fprintf(slfp, "%s Connection to %s (%s) refused\n", \
+               (*statstr)->_PFTP_DAEMON_ ? time_string(): "**", \
+               (*(statstr+strnum))->REMOTEHOSTNAME, \
+               inet_ntoa(fsin->sin_addr));
+#endif
             }
             close((*(statstr+strnum))->ns);
 #ifdef USE_POSIX_THREAD
@@ -453,6 +448,7 @@ void login_handle(void *structure_number)
 #endif
          return;
       }
+#if !defined __RH_CYGLINT__
       /*
        * If running as a daemon or
        * server started by inetd...
@@ -495,6 +491,7 @@ void login_handle(void *structure_number)
          }
       }
       else {
+#endif
          /*
           * Look if host is set in host name list.
           */
@@ -506,7 +503,9 @@ void login_handle(void *structure_number)
          if (goon && write((*(statstr+strnum))->ws, "STAT server\n", 12) < 0) {
             if (slfp) fprintf(slfp, "** write: %s\n", _PFTP_ERROR_ARRAY_);
          }
+#if !defined __RH_CYGLINT__
       }
+#endif
 #ifdef USE_POSIX_THREAD
       CHILDNUM++;
 #endif
