@@ -1,7 +1,7 @@
 /*
  * pftp -- sends files from host to host through free choosable ports
  *
- * Copyright (C) 1996-1999 Ben Schluricke
+ * Copyright (C) 1996-2000 Ben Schluricke
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,6 +41,10 @@
 #if defined Linux || defined NEXTSTEP
 #include <sys/dir.h>
 #endif
+#if defined OSF1
+#include <sys/stat.h>
+extern int mknod(const char *, int, dev_t);
+#endif
 #include "main.h"
 
 extern long dlastentry;
@@ -137,7 +141,7 @@ int set_status_information(char *file, struct stat *buf)
 
 short copy_file(char *s, char *d)
 {
-   int sfd=0, dfd=0;
+   int sfd=0, dfd=0, ptrun=(*statstr)->OVERWRITE ? O_TRUNC: 0;
    ssize_t n=0, check=0;
    extern char *saving_str;
    char str[BUFSIZE], c=0;
@@ -148,6 +152,7 @@ short copy_file(char *s, char *d)
    stmp = d;
    while (!access(stmp, F_OK) && !(*statstr)->OVERWRITE) {
       if ((*statstr)->_SKIP_) return 1;
+      ptrun = 0;
       fprintf(stderr, "** File %s exists!\n** Overwrite(y|n) all(Y), rename(r), skip(s) all(S), cancel(c) ? ", stmp);
       c = fgetc(stdin);
       fprintf(stderr, "\n%s", saving_str);
@@ -163,6 +168,7 @@ short copy_file(char *s, char *d)
             }
          }
          if (c == 'Y') (*statstr)->OVERWRITE = BIT_ONE;
+         ptrun = O_TRUNC;
          break;
       }
       else if (c == 'r') {
@@ -223,7 +229,7 @@ short copy_file(char *s, char *d)
             return 0;
          }
 
-         if ((dfd = open(stmp, O_CREAT|O_WRONLY, 0200)) < 0) {
+         if ((dfd = open(stmp, O_CREAT|O_WRONLY|ptrun, 0200)) < 0) {
             print_error_mesg("open", stmp);
             close(sfd);
             return 0;
@@ -410,48 +416,6 @@ char *get_subject(char *subj, char *user, char *date)
    return NULL;
 }
 
-#define WHELPLINES 14
-
-void warranty(void) {
-   char *firstline=NULL, *blankline=NULL, *ctmp=NULL;
-   char *warrantystr[WHELPLINES+1] = {
-      "Copyright (C) 1996-1999 Ben Schluricke                                 \n",
-      "                                                                       \n",
-      "This program is free software; you can redistribute it and/or modify   \n",
-      "it under the terms of the GNU General Public License as published by   \n",
-      "the Free Software Foundation; either version 2 of the License, or (at  \n",
-      "your option) any later version.                                        \n",
-      "                                                                       \n",
-      "This program is distributed in the hope that it will be useful, but    \n",
-      "WITHOUT ANY WARRANTY; without even the emplied warranty of MERCHANT-   \n",
-      "ABILITY OF FITNESS FOR A PARTICULAR PURPOSE. See the GNU General       \n",
-      "Public License for more details.                                       \n",
-      "                                                                       \n",
-      "You should have received a copy of the GNU General Public License      \n",
-      "along with this program; if not, write to the Free Software Foundation,\n",
-      "Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                          \n"
-   };
-
-   MEM_CHECK((firstline = (char *)calloc(LONAME, sizeof(char))));
-   sprintf(firstline, "****** Port-FTP Version %s Copyright (C) 1996-1999 Ben Schluricke ", VERSION); 
-   for (ctmp=firstline; *ctmp; ctmp++);
-   for (; ctmp - firstline < _WINCOLS_; ctmp++) *ctmp = '*';
-   *(ctmp-1) = ' ';
-   *ctmp = '\0';
-   MEM_CHECK((blankline = (char *)calloc(SONAME, sizeof(char))));
-   for (ctmp=blankline; ctmp - blankline < _WINCOLS_; ctmp++) *ctmp = ' ';
-   *ctmp='\r';
-   *++ctmp='\0';
-   
-   fprintf(stdout, "[H[J");
-   ctmp = headline;
-   headline = firstline;
-   display_pa(warrantystr, firstline, blankline, WHELPLINES);  
-   headline = ctmp;
-   free(blankline);
-   free(firstline);
-}
-
 
 int is_pftp_mesg(char *file)
 {
