@@ -1,7 +1,7 @@
 /*
  * pftp -- sends files from host to host through free choosable ports
  *
- * Copyright (C) 1996-1999 Ben Schluricke
+ * Copyright (C) 1996-2000 Ben Schluricke
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@
 #endif
 #include "main.h"
 
-extern void bubble_sort(char **, char *, long);
+extern void quick_sort(char **, char *, long);
 char *list_dir(char *, char *);
 extern char PFTP_SORT_BY;
 
@@ -89,13 +89,17 @@ char *tabdir(char *str)
                strcpy(tmp, cdir);
             }
             else if (*cdir) {
+#if !defined __RH_CYGLINT__
                if (*cdir == '~' && getpwuid(getuid())) {
                   sprintf(tmp, "%s%s%s", (char *)getpwuid(getuid())->pw_dir, \
                   *(cdir+1) == '/'? "/": "", *(cdir+2)? cdir+2: "");
                }
                else {
+#endif
                   sprintf(tmp, "%s%s%s", str, strcmp(str, "/")? "/": "", cdir);
+#if !defined __RH_CYGLINT__
                }
+#endif
             }
             else strcpy(tmp, str);
             if (*cdir) {
@@ -249,7 +253,7 @@ char *list_dir(char *directory, char *mask)
        */
       ch = PFTP_SORT_BY;
       PFTP_SORT_BY = 'a';
-      bubble_sort(direntries, directory, (long) endnum);
+      quick_sort(direntries, directory, (long) endnum);
       PFTP_SORT_BY = ch;
       
       for (maxlen=i=0; i < endnum; i++) {
